# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys
import code
import readline
import __builtin__

from LMIUtil import LMIPassByRef
from LMIIndication import LMIIndication
from LMIIndicationListener import LMIIndicationListener
from LMIBaseClient import LMIBaseClient
from LMIShellClient import LMIShellClient
from LMIShellConfig import LMIShellConfig
from LMIShellOptions import LMIShellOptions
from LMISubscription import LMISubscription
from LMICompleter import LMICompleter
from LMIConnection import LMIConnection
from LMIShellConfig import LMIShellConfig
from LMIReturnValue import LMIReturnValue
from LMIObjectFactory import LMIObjectFactory

from LMIConnection import connect as connect_internal

from LMIUtil import lmi_set_use_exceptions
from LMIUtil import lmi_get_use_exceptions
from LMIUtil import lmi_associators
from LMIUtil import lmi_isinstance

from LMIHelper import LMIHelper

class LMIConsole(code.InteractiveConsole):
    """
    Class representing an interactive console.
    """
    SHELL_PS1 = "> "
    SHELL_PS2 = "... "
    DEFAULT_LOCALS = {
        "__name__" : "__main__",
        "LMIPassByRef" : LMIPassByRef,
        "LMIIndication" : LMIIndication,
        "LMIIndicationListener" : LMIIndicationListener,
        "LMIReturnValue" : LMIReturnValue,
        "LMIBaseClient" : LMIBaseClient,
        "LMIShellClient" : LMIShellClient,
        "LMIShellConfig" : LMIShellConfig,
        "LMIShellOptions" : LMIShellOptions,
        "LMISubscription" : LMISubscription,
        "LMIConnection" : LMIConnection,
        "LMINamespace" : LMIObjectFactory().LMINamespace,
        "LMINamespaceRoot" : LMIObjectFactory().LMINamespaceRoot,
        "LMIClass" : LMIObjectFactory().LMIClass,
        "LMIInstance" : LMIObjectFactory().LMIInstance,
        "LMIInstanceName" : LMIObjectFactory().LMIInstanceName,
        "LMIMethod" : LMIObjectFactory().LMIMethod,
        "use_exceptions" : lmi_set_use_exceptions,
        "lmi_associators" : lmi_associators,
        "lmi_isinstance" : lmi_isinstance,
        "help" : LMIHelper()
    }

    def __init__(self):
        # Setup default LMIConsole locals, that are needed
        # for proper object construction
        locals = dict(LMIConsole.DEFAULT_LOCALS.items())
        code.InteractiveConsole.__init__(self, locals)

        # Store configuration
        config = LMIShellConfig()
        self._history_file = config.history_file
        self._history_length = config.history_length
        self._use_cache = config.use_cache
        self._use_exceptions = config.use_exceptions
        self._verify_server_cert = True

        # Setup LMIShell-wide option, which defines, whether the LMIShell
        # should propagate caught exceptions, or dump them
        lmi_set_use_exceptions(self._use_exceptions)

    def setup_completer(self):
        """
        Initializes tab-completer.
        """
        self._completer = LMICompleter(self.locals)
        readline.set_completer(self._completer.complete)
        readline.parse_and_bind("tab: complete")

    def interact(self, locals=None):
        """
        Starts the interactive mode.

        :param dictionary locals: locals
        """
        def connect(uri, username="", password="", **kwargs):
            """
            Returns :py:class:`LMIConnection` object with provided URI and
            credentials.

            :param string uri: URI of the CIMOM
            :param string username: account, under which, the CIM calls will be performed
            :param string password: user's password
            :param dictionary kwargs: supported keyword arguments

                * **key_file** (*string*) -- path to x509 key file; default value is None
                * **cert_file** (*string*) -- path to x509 cert file; default value is None
                * **verify_server_cert** (*bool*) -- tells, if the server side certificate
                  needs to be verified, if SSL used; default value is True

            :returns: connection object
            :rtype: :py:class:`LMIConnection`
            """
            key_file = kwargs.pop("key_file", None)
            cert_file = kwargs.pop("cert_file", None)
            verify_server_cert = kwargs.pop("verify_server_cert", self._verify_server_cert)
            if kwargs:
                raise TypeError("connect() got an unexpected keyword arguments: %s" % ", ".join(kwargs.keys()))

            return connect_internal(uri, username, password, interactive=True,
                use_cache=self._use_cache, key_file=key_file, cert_file=cert_file,
                verify_server_cert=verify_server_cert)

        # Initialize the interpreter
        if locals is None:
            locals = {}
        else:
            for (k, v) in locals.iteritems():
                if isinstance(v, LMIConnection):
                    locals[k]._client.interactive = True
        locals["clear_history"] = self.clear_history
        locals["connect"] = connect
        self.locals = dict(self.locals.items() + locals.items())
        self.setup_completer()
        self.load_history()
        old_sys_ps1 = ""
        try:
            old_sys_ps1 = sys.ps1
        except AttributeError, e:
            old_sys_ps1 = ""
        try:
            old_sys_ps2 = sys.ps2
        except AttributeError, e:
            old_sys_ps2 = ""
        sys.ps1 = LMIConsole.SHELL_PS1
        sys.ps2 = LMIConsole.SHELL_PS2

        # Interact
        more = 0
        while 1:
            try:
                if more:
                    prompt = sys.ps2
                else:
                    prompt = sys.ps1
                try:
                    line = self.raw_input(prompt)
                    # Can be None if sys.stdin was redefined
                    encoding = getattr(sys.stdin, "encoding", None)
                    if encoding and not isinstance(line, unicode):
                        line = line.decode(encoding)
                except EOFError:
                    self.write("\n")
                    break
                else:
                    more = self.push(line)
            except KeyboardInterrupt:
                self.write("\n")
                self.resetbuffer()
                more = 0

        # Cleanup after the interpreter
        if old_sys_ps1:
            sys.ps1 = old_sys_ps1
        if old_sys_ps2:
            sys.ps2 = old_sys_ps2
        self.save_history()

    def interpret(self, script_name, script_argv, locals=None):
        """
        Interprets a specified script within additional provided locals.
        There are :py:attr:`LMIConsole.DEFAULT_LOCALS` present.

        :param string script_name: script name
        :param list script_argv: script CLI arguments
        :param dictionary locals: dictionary with locals
        :returns: exit code of the script
        :rtype: int
        """
        # Helper function for non-interactive connect
        def connect(uri, username="", password="", **kwargs):
            """
            Returns :py:class:`LMIConnection` object with provided URI and credentials.

            :param string uri: URI of the CIMOM
            :param string username: account, under which, the CIM calls will be performed
            :param string password: user's password
            :param dictionary kwargs: supported keyword arguments

                * **key_file** (*string*) -- path to x509 key file; default value is None
                * **cert_file** (*string*) -- path to x509 cert file; default value is None
                * **verify_server_cert** (*bool*) -- tells, if the server side certificate
                  needs to be verified, if SSL used; default value is True

            :returns: connection object
            :rtype: :py:class:`LMIConnection`
            """
            # Set remaining arguments
            key_file = kwargs.pop("key_file", None)
            cert_file = kwargs.pop("cert_file", None)
            verify_server_cert = kwargs.pop("verify_server_cert", self._verify_server_cert)
            if kwargs:
                raise TypeError("connect() got an unexpected keyword arguments: %s" % ", ".join(kwargs.keys()))

            return connect_internal(uri, username, password, interactive=False,
                use_cache=self._use_cache, key_file=key_file, cert_file=cert_file,
                verify_server_cert=verify_server_cert)
        # Initialize locals
        if locals is None:
            locals = {}
        else:
            for (k, v) in locals.iteritems():
                if isinstance(v, LMIConnection):
                    locals[k]._client.interactive = False
        locals["connect"] = connect
        self.locals = dict(self.locals.items() + locals.items())

        # Execute the script
        sys.argv = script_argv
        exit_code = 0
        try:
            execfile(script_name, self.locals)
        except SystemExit, e:
            exit_code = e.code

        # Return with exit_code
        return exit_code

    def load_history(self):
        """
        Loads the shell's history from the history file.
        """
        if self._history_length == 0 or not os.path.exists(self._history_file):
            return
        readline.read_history_file(self._history_file)
        if self._history_length > 0 and readline.get_current_history_length() > self._history_length:
            readline.set_history_length(self._history_length)
            readline.write_history_file(self._history_file)
            readline.read_history_file(self._history_file)

    def save_history(self):
        """
        Saves current history of commands into the history file. If the length of history
        exceeds a maximum history file length, the history will be truncated.
        """
        if self._history_length == 0:
            return
        elif self._history_length > 0:
            readline.set_history_length(self._history_length)
        try:
            readline.write_history_file(self._history_file)
        except IOError, e:
            pass

    def clear_history(self):
        """
        Clears the current history.
        """
        readline.clear_history()

    def set_verify_server_certificate(self, verify_server_cert=True):
        """
        Turns on or off server side certificate verification, if SSL used.

        :param bool verify_server_cert: -- flag which tells, whether a server side
            certificate needs to be verified, if SSL used
        """
        self._verify_server_cert = verify_server_cert



# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import abc
import sys
import pywbem
import tempfile
import subprocess
from textwrap import TextWrapper
from xml.dom.minicompat import NodeList

from LMIUtil import lmi_raise_or_dump_exception

from LMIExceptions import LMINoPagerError

def _is_executable(fpath):
    """
    :param string fpath: path of executable
    :returns: True, if provided path is executable, False otherwise
    """
    return os.path.exists(fpath) and os.access(fpath, os.X_OK)

def _which(program):
    """
    :param string program: executable name
    :returns: full path of a selected executable
    :rtype: string
    """
    (fpath, fname) = os.path.split(program)
    if fpath:
        if _is_executable(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if _is_executable(exe_file):
                return exe_file
    return None

def _get_pager_with_params():
    """
    :returns: list containing a executable with its CLI arguments
    """
    if "PAGER" in os.environ:
        path = _which(os.environ["PAGER"])
        if path:
            return [path]
    for p in ["less", "more"]:
        path = _which(p)
        if not path:
            continue
        elif p == "less":
            return [path, "-S"]
        return [path]
    lmi_raise_or_dump_exception(LMINoPagerError("No default pager found"))
    return []

class LMITextFormatter(object):
    """
    Text formatter class. Used when printing a block of text to output stream.

    :param string text: text to be formatted
    """
    def __init__(self, text):
        self._text = text

    def format(self, indent=0, sub_indent=0, width=80, f=sys.stdout, **kwargs):
        """
        Formats a block of text and prints it to the output stream.

        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: output stream
        :param dictionary kwargs: supported keyword arguments

            * **separator** (*bool*) -- if True, there will be a new line appended after
              the formatted text
        """
        if indent > width:
            return # NOTE: this is wrong!
        wrapper = TextWrapper()
        wrapper.width = width - indent
        wrapper.subsequent_indent = " " * sub_indent
        for l in wrapper.wrap(self._text):
            f.write("%s%s\n" % (" " * indent, l))
        if "separator" in kwargs and kwargs["separator"]:
            f.write("\n")

class LMIFormatter(object):
    """
    Abstract class for XML formatter and MOF formatter.
    """
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        stty_dimensions = os.popen("stty size 2> /dev/null", "r").read().split()
        self._width = int(stty_dimensions[1]) if stty_dimensions else 80

    @abc.abstractmethod
    def format(self, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a block of text and prints it to the output stream.

        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: output stream
        """
        pass

    def fancy_format(self, interactive):
        """
        Formats a block of text. If the LMIShell is running in interactive mode, pager
        will be used, otherwise the output will be written to standard output.

        :param bool interactive: defines, if to use pager
        """
        if interactive:
            tmpfile = tempfile.mkstemp()
            f = os.fdopen(tmpfile[0], "w")
            self.format(0, 0, self._width, f)
            f.close()
            subprocess_params = _get_pager_with_params()
            if not subprocess_params:
                return
            subprocess_params.append(tmpfile[1])
            subprocess.call(subprocess_params)
            os.remove(tmpfile[1])
        else:
            self.format(0, 0, self._width)

class LMIXmlFormatter(LMIFormatter):
    """
    XML formatter used for :py:meth:`CIMClass.doc` to print pretty verbose help message.

    :param pywbem.cim_xml.CLASS xml: -- class element
    """
    def __init__(self, xml):
        super(LMIXmlFormatter, self).__init__()
        self._xml = xml

    def __format_class_property_content(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a class property.

        :param pywbem.cim_xml.PROPERTY node: property element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        children = node.childNodes
        children_len = len(children)
        for (i, n) in enumerate(children):
            if isinstance(n, pywbem.cim_xml.QUALIFIER):
                self.__format_qualifier(n, indent, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.PROPERTY):
                self.__format_class_property(n, indent, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.PROPERTY_ARRAY):
                self.__format_class_property_array(n, indent, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.METHOD):
                self.__format_method(n, indent, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.VALUE):
                # NOTE: we skip default value for the class
                pass

    def __format_qualifier(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a qualifier.

        :param pywbem.cim_xml.QUALIFIER node: qualifier element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        is_array = isinstance(node.firstChild, pywbem.cim_xml.VALUE_ARRAY)
        val = "[qualifier] %s%s %s" % (node.getAttribute("TYPE"), " []" if is_array else "",
            node.getAttribute("NAME"))
        values = node.childNodes
        if values and isinstance(values[0], pywbem.cim_xml.VALUE_ARRAY):
            val += ": { " + ", ".join(["'" + v.firstChild.nodeValue + "'" for v in values[0].childNodes]) + " }\n"
        else:
            if values:
                val += ": '%s'\n" % values[0].firstChild.nodeValue
        LMITextFormatter(val).format(indent, sub_indent + 4, width, f, separator=True)

    def __format_class_property(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a class property.

        :param pywbem.cim_xml.PROPERTY node: property element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        val = "[property] %s %s" % (node.getAttribute("TYPE"), node.getAttribute("NAME"))
        LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)
        self.__format_class_property_content(node, indent+4, sub_indent, width, f)

    def __format_class_property_array(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a class property array.

        :param pywbem.cim_xml.PROPERTY_ARRAY node: property array element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        val = "[property array] %s [] %s" % (node.getAttribute("TYPE"), node.getAttribute("TYPE"))
        LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)
        self.__format_class_property_content(node, indent+4, sub_indent, width, f)

    def __format_class_property_reference(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a class property reference.

        :param pywbem.cim_xml.PROPERTY_REFERENCE node: property reference element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        val = "[property ref] %s %s" % (node.getAttribute("REFERENCECLASS"), node.getAttribute("NAME"))
        LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)
        self.__format_class_property_content(node, indent+4, sub_indent, width, f)

    def __format_instance_property(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a instance property.

        :param pywbem.cim_xml.PROPERTY node: property element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        val = node.firstChild
        prop = "[property] %s %s%s" % (node.getAttribute("TYPE"), node.getAttribute("NAME"),
            " = '%s'" % val.firstChild.nodeValue if val else "")
        LMITextFormatter(prop).format(indent, sub_indent+4, width, f, separator=True)

    def __format_instance_property_array(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a instance property array.

        :param pywbem.cim_xml.PROPERTY_ARRAY node: property array element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        prop = "[property array] %s [] %s" % (node.getAttribute("TYPE"), node.getAttribute("NAME"))
        val = node.firstChild
        if val:
            prop += " = { " + ", ".join(["'" + v.firstChild.nodeValue + "'" for v in val.childNodes]) + " }"
        LMITextFormatter(prop).format(indent, sub_indent+4, width, f, separator=True)

    def __format_instance_property_reference(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a instance property reference.

        :param pywbem.cim_xml.PROPERTY_REFERENCE node: property reference element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        val = "[property ref] %s %s" % (node.getAttribute("REFERENCECLASS"), node.getAttribute("NAME"))
        LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)

    def __format_method(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a class method.

        :param pywbem.cim_xml.METHOD node: method element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: output stream
        """
        qualifiers = [x for x in node.childNodes if isinstance(x, pywbem.cim_xml.QUALIFIER)]
        parameters = [x for x in node.childNodes if isinstance(x, pywbem.cim_xml.PARAMETER)]
        parameters_arr = [x for x in node.childNodes if isinstance(x, pywbem.cim_xml.PARAMETER_ARRAY)]
        parameters_ref = [x for x in node.childNodes if isinstance(x, pywbem.cim_xml.PARAMETER_REFERENCE)]
        parameters_ref_arr = [x for x in node.childNodes if isinstance(x, pywbem.cim_xml.PARAMETER_REFARRAY)]
        has_args = parameters != [] or parameters_arr != [] or parameters_ref != [] or parameters_ref_arr != []
        val = "[method] %s %s%s" % (node.getAttribute("TYPE"), node.getAttribute("NAME"), "(...)" if has_args else "()")
        LMITextFormatter(val).format(indent, sub_indent, width, f, separator=True)
        for q in qualifiers:
            self.__format_qualifier(q, indent+4, sub_indent+4, width, f)
        for p in parameters:
            val = "[param] %s %s" % (p.getAttribute("TYPE"), p.getAttribute("NAME"))
            LMITextFormatter(val).format(indent+4, sub_indent+4, width, f, separator=True)
        for p in parameters_arr:
            val = "[param array] %s (ref) %s" % (p.getAttribute("TYPE"), p.getAttribute("NAME"))
            LMITextFormatter(val).format(indent+4, sub_indent+4, width, f, separator=True)
        for p in parameters_ref:
            val = "[param ref] %s (ref) %s" % (p.getAttribute("REFERENCECLASS"), p.getAttribute("NAME"))
            LMITextFormatter(val).format(indent+4, sub_indent+4, width, f, separator=True)
        for p in parameters_ref_arr:
            val = "[param ref array] %s (ref) %s" % (p.getAttribute("REFERENCECLASS"), p.getAttribute("NAME"))
            LMITextFormatter(val).format(indent+4, sub_indent+4, width, f, separator=True)

    def __format_class(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a class.

        :param pywbem.cim_xml.CLASS node: class element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: output stream
        """
        LMITextFormatter("Class: %s" % node.getAttribute("NAME")).format(indent, sub_indent, width, f)
        LMITextFormatter("SuperClass: %s" % node.getAttribute("SUPERCLASS")).format(indent+4, sub_indent, width, f)
        for n in node.childNodes:
            if isinstance(n, pywbem.cim_xml.QUALIFIER):
                self.__format_qualifier(n, indent+4, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.PROPERTY):
                self.__format_class_property(n, indent+4, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.PROPERTY_ARRAY):
                self.__format_class_property_array(n, indent+4, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.PROPERTY_REFERENCE):
                self.__format_class_property_reference(n, indent+4, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.METHOD):
                self.__format_method(n, indent+4, sub_indent, width, f)

    def __format_instance(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats an instance.

        :param pywbem.cim_xml.VALUE_NAMEDINSTANCE node: instance element
        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: -- output stream
        """
        instance = node.firstChild.nextSibling
        LMITextFormatter("Instance of %s" % instance.getAttribute("CLASSNAME")).format(indent, sub_indent, width, f)
        for n in instance.childNodes:
            if isinstance(n, pywbem.cim_xml.PROPERTY):
                self.__format_instance_property(n, indent+4, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.PROPERTY_ARRAY):
                self.__format_instance_property_array(n, indent+4, sub_indent, width, f)
            elif isinstance(n, pywbem.cim_xml.PROPERTY_REFERENCE):
                self.__format_instance_property_reference(n, indent+4, sub_indent, width, f)

    def format(self, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a XML object and prints it to the output stream.

        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: output stream
        """
        if isinstance(self._xml, pywbem.cim_xml.CLASS):
            self.__format_class(self._xml, indent, sub_indent, width, f)
        elif isinstance(self._xml, pywbem.cim_xml.VALUE_NAMEDINSTANCE):
            self.__format_instance(self._xml, indent, sub_indent, width, f)

class LMIMofFormatter(LMIFormatter):
    """
    MOF formatter used for :py:meth:`CIMInstance.doc` to print pretty verbose help
    message.

    :param string mof: MOF representation of a object
    """
    def __init__(self, mof):
        super(LMIMofFormatter, self).__init__()
        self._mof = mof

    def format(self, indent=0, sub_indent=0, width=80, f=sys.stdout):
        """
        Formats a MOF object and prints it to the output stream.

        :param int indent: number of spaces to indent the text block
        :param int sub_indent: number of spaces for the second and other lines
            of the text block
        :param int width: total text block width
        :param f: output stream
        """
        f.write(self._mof)



# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys
import hashlib
import pywbem

from LMIObjectFactory import LMIObjectFactory

# By default, the LMIShell does not use exceptions; LMIReturnValue
# is used instead (with proper error string).
class LMIUseExceptionsHelper(object):
    """
    Singleton helper class used for storing a bool flag, which defines,
    if the LMIShell should propagate exceptions or dump them.
    """
    _instance = None

    def __new__(cls):
        """
        Return a new :py:class:`LMIUseExceptionsHelper` instance, if no shared object is
        present; otherwise an existing instance is returned. By default the
        ``use_exceptions`` flag is set to False.
        """
        if cls._instance is None:
            cls._instance = super(LMIUseExceptionsHelper, cls).__new__(cls)
            cls._instance._use_exceptions = False
        return cls._instance

    @property
    def use_exceptions(self):
        """
        :returns: whether the LMIShell should propagate the exceptions, or throw them away
        :rtype: bool
        """
        return self._use_exceptions

    @use_exceptions.setter
    def use_exceptions(self, use=True):
        """
        Property setter, which modifies the bool flag, which indicates, if the LMIShell
        should propagate the exceptions, or dump them.

        :param bool use: specifies, whether to use exceptions in LMIShell
        """
        self._use_exceptions = use

class LMIPassByRef(object):
    """
    Helper class used for passing a value by reference. It uses the advantage of python,
    where all the dictionaries are passed by reference.

    :param val: value, which will be passed by reference

    Example of usage:

    .. code-block:: python

        by_ref = LMIPassByRef(some_value)
        by_ref.value == some_value
    """
    def __init__(self, val):
        self._val = {0 : val}

    @property
    def value(self):
        """
        :returns: value passed by reference.
        """
        return self._val[0]

    @value.setter
    def value(self, new_val):
        """
        Property setter for the value passed by reference.

        :param new_val: new value, which is passed by reference
        """
        self._val[0] = new_val

def lmi_get_use_exceptions():
    """
    :returns: whether the LMIShell should use the exceptions, or throw them away
    :rtype: bool
    """
    return LMIUseExceptionsHelper().use_exceptions

def lmi_set_use_exceptions(use=True):
    """
    Sets a global flag indicating, if the LMIShell should use the exceptions, or throw
    them away.

    :param bool use: specifies, whether the LMIShell should use the exceptions
    """
    LMIUseExceptionsHelper().use_exceptions = use

def lmi_raise_or_dump_exception(e=None):
    """
    Function which either raises an exception, or throws it away.

    :param Exception e: exception, which will be either raised or thrown away
    """
    if not lmi_get_use_exceptions():
        return
    (et, ei, tb) = sys.exc_info()
    if e is None:
        raise et, ei, tb
    else:
        raise type(e), e, tb

def _lmi_do_cast(t, value, cast):
    """
    Helper function, which preforms the actual cast.

    :param string t: string of CIM type
    :param value: variable to cast
    :param dictionary cast: dictionary with :samp:`type : cast_func`
    :returns: cast value
    """
    cast_func = cast.get(t.lower(), lambda x: x)
    if isinstance(value, (dict, pywbem.NocaseDict)):
        return pywbem.NocaseDict({
            k: _lmi_do_cast(t, val, cast) for k, val in value.iteritems()})
    elif isinstance(value, list):
        return map(lambda val: _lmi_do_cast(t, val, cast), value)
    elif isinstance(value, tuple):
        return tuple(map(lambda val: _lmi_do_cast(t, val, cast), value))
    return cast_func(value)

def lmi_cast_to_cim(t, value):
    """
    Casts the value to CIM type.

    :param string t: string of CIM type
    :param value: variable to cast
    :returns: cast value in :py:mod:`pywbem` type
    """
    cast = {
        "sint8"  : lambda x: pywbem.Sint8(x),
        "uint8"  : lambda x: pywbem.Uint8(x),
        "sint16" : lambda x: pywbem.Sint16(x),
        "uint16" : lambda x: pywbem.Uint16(x),
        "sint32" : lambda x: pywbem.Sint32(x),
        "uint32" : lambda x: pywbem.Uint32(x),
        "sint64" : lambda x: pywbem.Sint64(x),
        "uint64" : lambda x: pywbem.Uint64(x),
        "reference": lambda x: x.path if isinstance(x, LMIObjectFactory().LMIInstance) else x
    }
    return _lmi_do_cast(t, value, cast)

def lmi_cast_to_lmi(t, value):
    """
    Casts the value to LMI (python) type.

    :param string t: string of CIM type
    :param value: variable to cast
    :returns: cast value in :py:mod:`python` native type
    """
    cast = {
        "sint8"  : lambda x: int(x),
        "uint8"  : lambda x: int(x),
        "sint16" : lambda x: int(x),
        "uint16" : lambda x: int(x),
        "sint32" : lambda x: int(x),
        "uint32" : lambda x: int(x),
        "sint64" : lambda x: int(x),
        "uint64" : lambda x: int(x),
    }
    return _lmi_do_cast(t, value, cast)

def lmi_wrap_cim_namespace(conn, cim_namespace_name):
    """
    Helper function, which returns wrapped CIM namespace in :py:class:`LMINamespace`.

    :param LMIConnection conn: connection object
    :param string cim_namespace_name: CIM namespace name
    :returns: wrapped CIM namespace into :py:class:`LMINamespace`
    """
    return LMIObjectFactory().LMINamespace(conn, cim_namespace_name)

def lmi_wrap_cim_class(conn, cim_class_name, cim_namespace_name):
    """
    Helper function, which returns wrapped :py:class:`CIMClass` into LMIClass.

    :param LMIConnection conn: connection object
    :param string cim_class_name: string containing :py:class:`CIMClass` name
    :param string cim_namespace_name: string containing :py:class:`CIMNamespace` name, or
        None, if the namespace is not known
    :returns: wrapped :py:class:`CIMClass` into :py:class:`LMIClass`
    """
    lmi_namespace = None
    if cim_namespace_name:
        lmi_namespace = lmi_wrap_cim_namespace(conn, cim_namespace_name)
    return LMIObjectFactory().LMIClass(conn, lmi_namespace, cim_class_name)

def lmi_wrap_cim_instance(conn, cim_instance, cim_class_name, cim_namespace_name):
    """
    Helper function, which returns wrapped :py:class:`CIMInstance` into
    :py:class:`LMIInstance`.

    :param LMIConnection conn: connection object
    :param CIMInstance cim_instance: :py:class:`CIMInstance` object to be wrapped
    :param string cim_class_name: :py:class:`CIMClass` name
    :param string cim_namespace_name: :py:class:`CIMNamespace` name, or None, if the
        namespace is not known
    :returns: wrapped :py:class:`CIMInstance` into :py:class:`LMIInstance`
    """
    lmi_class = lmi_wrap_cim_class(conn, cim_class_name, cim_namespace_name)
    return LMIObjectFactory().LMIInstance(conn, lmi_class, cim_instance)

def lmi_wrap_cim_instance_name(conn, cim_instance_name):
    """
    Helper function, which returns wrapped :py:class:`CIMInstanceName` into
    :py:class:`LMIInstanceName`.

    :param LMIConnection conn: connection object
    :param CIMInstanceName cim_instance_name: :py:class:`CIMInstanceName` object to be
        wrapped
    :returns: wrapped :py:class:`CIMInstanceName` into :py:class:`LMIInstanceName`
    """
    return LMIObjectFactory().LMIInstanceName(conn, cim_instance_name)

def lmi_wrap_cim_method(conn, cim_method_name, lmi_instance, sync_method):
    """
    Helper function, which returns wrapped :py:class:`CIMMethod` into
    :py:class:`LMIMethod`.

    :param LMIConnection conn: connection object
    :param string cim_method_name: method name
    :param LMIInstance lmi_instance: object, on which the method call will be issued
    :param bool sync_method: flag indicating, if we are trying to perform
        a synchronous method call
    :returns: wrapped :py:class:`CIMMethod` into :py:class:`LMIMethod`
    """
    return LMIObjectFactory().LMIMethod(conn, lmi_instance, cim_method_name, sync_method)

def lmi_transform_to_lmi(conn, value):
    """
    Transforms returned values from a method call into LMI wrapped objects. Returns
    transformed input, where :py:class:`CIMInstance` and :py:class:`CIMInstanceName` are
    wrapped into LMI wrapper classes and primitive types are cast to python native types.

    :param LMIConnection conn: connection object
    :param value: object to be transformed into :py:mod:`python` type from :mod:`pywbem` one
    :returns: transformed py::mod:`pywbem` object into LMIShell one
    """
    if isinstance(value, pywbem.cim_obj.CIMInstance):
        namespace = value.path.namespace if value.path else None
        return lmi_wrap_cim_instance(conn, value, value.classname, namespace)
    elif isinstance(value, pywbem.cim_obj.CIMInstanceName):
        return lmi_wrap_cim_instance_name(conn, value)
    elif isinstance(value, pywbem.CIMInt):
        return int(value)
    elif isinstance(value, pywbem.CIMFloat):
        return float(value)
    elif isinstance(value, (dict, pywbem.NocaseDict)):
        return pywbem.NocaseDict({
                k: lmi_transform_to_lmi(conn, val)
            for k, val in value.iteritems()})
    elif isinstance(value, list):
        return map(lambda val: lmi_transform_to_lmi(conn, val), value)
    elif isinstance(value, tuple):
        return tuple(map(lambda val: lmi_transform_to_lmi(conn, val), value))
    return value

def lmi_isinstance(lmi_obj, lmi_class):
    """
    Function returns True if :samp:`lmi_obj` is an instance of a :samp:`lmi_class`, False
    otherwise. When passed :py:class:`LMIInstance`, :py:class:`LMIInstanceName` as
    :samp:`lmi_obj` and :samp:`lmi_class` is of :py:class:`LMIClass` type, function can
    tell, if such :samp:`lmi_obj` is direct instance of :py:class:`LMIClass`, or it's
    super class.

    If :samp:`lmi_obj` and :samp:`lmi_class` is not instance of mentioned classes, an
    exception will be raised.

    :param lmi_obj: instance of :py:class:`LMIInstance` or :py:class:`LMIInstanceName`
        which is checked, if such instance is instance of the ``lmi_class``
    :param LMIClass lmi_class: instance of :py:class:`LMIClass` object
    :returns: whether **lmi_obj** is instance of **lmi_class**
    :rtype: bool
    :raises: :py:exc:`TypeError`
    """
    if not isinstance(lmi_obj, (LMIObjectFactory().LMIInstance,
            LMIObjectFactory().LMIInstanceName)) or \
            not isinstance(lmi_class, LMIObjectFactory().LMIClass):
        errorstr = "Use with types LMIInstance/LMIInstanceName and LMIClass"
        lmi_raise_or_dump_exception(TypeError(errorstr))
        return False
    client = lmi_obj._conn._client
    classname = lmi_obj.classname
    namespace = lmi_obj.namespace
    while classname:
        if classname == lmi_class.classname:
            return True
        classname = client._get_superclass(classname, namespace)
    return False

def lmi_script_name():
    """
    :returns: script name
    :rtype: string
    """
    return os.path.basename(sys.argv[0])

def lmi_associators(assoc_classes):
    """
    Helper function to speed up associator traversal. Returns a list of tuples, where each
    tuple contains :py:class:`LMIInstance` objects, which are in association.

    :param list assoc_classes: list of :py:class:`LMIClass` objects, for which the
        associations will be returned
    :returns: list of tuples of :py:class:`LMIInstance` objects in association
    """
    def make_key(path):
        path.host = None
        return hashlib.md5(path.classname.lower() + path.namespace.lower() + \
            str({ k.lower() : v for k, v in path.keybindings.iteritems() })).hexdigest()

    result = []

    instances = {}
    for assoc_class in assoc_classes:
        conn = assoc_class._conn
        assoc_class.fetch()

        # Reference properties
        ref_props = [
            ref_prop_name \
            for (ref_prop_name, ref_prop) in assoc_class._cim_class.properties.iteritems() \
                if ref_prop.type == "reference"
        ]

        # Reference class names
        ref_class_names = [
            assoc_class._cim_class.properties[ref_prop].reference_class for ref_prop in ref_props
        ]

        # Get instances, which will be joined as associators
        for ref_class_name in ref_class_names:
            (inst_list, out, err) = conn._client._get_instances(ref_class_name)
            instances.update({ make_key(inst.path) : inst for inst in inst_list })

        # Join associated objects
        (assoc_instance_names, out, err) = conn._client._get_instance_names(assoc_class.classname)
        for assoc in assoc_instance_names:
            ref_list = []
            for ref_prop in ref_props:
                path = assoc[ref_prop]
                # XXX: Some association classes report in key property a class, which its
                # instances do not refer to.
                inst = instances.get(make_key(path), None)
                if inst:
                    ref_list.append(
                        lmi_wrap_cim_instance(conn, inst, inst.classname,
                            inst.path.namespace)
                    )

            if len(ref_list) == len(ref_class_names):
                result.append(tuple(ref_list))

    return result



# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import sys
import ssl
import time
import pywbem
import random
import socket
import string
import threading

from SocketServer import BaseRequestHandler
from SocketServer import ThreadingMixIn
from BaseHTTPServer import HTTPServer
from BaseHTTPServer import BaseHTTPRequestHandler

from LMIIndication import LMIIndication

from LMIExceptions import LMIHandlerNamePatternError

class LMIIndicationHandlerCallback(object):
    """
    Helper class, which stores indication handler callback with its arguments and keyword
    arguments.

    :param callback: callback, which is called, when a indication arrives
    :param tuple args: positional arguments for callback
    :param kwargs: keyword arguments for callback
    """
    def __init__(self, callback, *args, **kwargs):
        self.callback = callback
        self.args = args
        self.kwargs = kwargs

class LMIIndicationHandler(ThreadingMixIn, BaseHTTPRequestHandler):
    """
    Class representing indication handler. The class is derived from
    :py:class:`BaseHTTPRequestHandler`, because the indications are transported by http
    protocol, and from :py:class:`ThreadingMixIn`; the indication listener is designed as
    concurrent server to properly handle each incoming indication.
    """
    def do_POST(self):
        """
        Overridden method, which is called, when a indication is received. It parses the
        indication message and searches for a matching handler for the indication name.
        Each subscribed indication should have it's Destination set something similar:

        ``<schema>://<destination>/<indication_name>``

        where the ``indication_name`` is a string, which properly defines the indication.
        """
        msg = self.rfile.read()
        tt = pywbem.parse_cim(pywbem.xml_to_tupletree(msg))
        message = tt[2]
        export_methods = {}
        if message[0].upper() != "MESSAGE":
            return
        message_params = message[2]
        if not message_params:
            return
        for param in message_params:
            if param[0].upper() != "SIMPLEEXPREQ":
                continue
            export_method_call = param[2]
            export_method_name = export_method_call[1]["NAME"]
            exp_params = export_method_call[2]
            export_method_params = {}
            for method_param in exp_params:
                export_method_params[method_param[0]] = method_param[1]
            export_methods[export_method_name] = export_method_params
        ind = LMIIndication(export_methods)
        path = self.path[1:]
        if path in self.server._handlers:
            cb = self.server._handlers[path]
            cb.callback(ind, *cb.args, **cb.kwargs)

class LMIIndicationServer(ThreadingMixIn, HTTPServer):
    """
    Class representing indication server, derived from HTTPServer and designed as
    concurrent server.
    """
    daemon_threads = True

class LMIIndicationListener(object):
    """
    Class representing indication listener, which provides a unified API for the server
    startup and shutdown and for registering an indication handler.

    :param string hostname: hostname or address of the machine, where the
        indications will be delivered
    :param int port: TCP port, where the server should listen for incoming messages
    :param string certfile: path to certificate file, if SSL used
    :param string keyfile: path to key file, if SSL used
    """
    # Minimum replacable "X" characters in handler pattern string.
    HANDLER_MINIMUM_REPL_CHARS_COUNT = 8

    def __init__(self, hostname, port, certfile=None, keyfile=None):
        self._handlers = {}
        self._hostname = hostname
        self._port = port
        self._certfile = certfile
        self._keyfile = keyfile

    def __create_handler_name(self, handler_name_pattern):
        """
        Returns unique handler name by replacing "**X**" characters for random characters
        at the end of the handler_name_pattern.

        :param string handler_name_pattern: string containing replaceable characters at the end
        :returns: unique handler name
        :rtype: string
        """
        x_cnt = 0
        pattern_len = len(handler_name_pattern)
        while True:
            while handler_name_pattern[pattern_len - 1 - x_cnt] == "X":
                x_cnt += 1
            if x_cnt < LMIIndicationListener.HANDLER_MINIMUM_REPL_CHARS_COUNT:
                raise LMIHandlerNamePatternError("Not enough replaceable characters provided")
            unique_str = handler_name_pattern[:pattern_len - x_cnt] + \
                "".join(random.choice(string.ascii_uppercase + string.digits) for x in range(x_cnt))
            if not unique_str in self._handlers:
                break
        return unique_str

    def start(self):
        """
        Starts a indication listener.

        :returns: True, if the indication listener started with no errors; False otherwise
        """
        try:
            self._server = LMIIndicationServer((self._hostname, self._port), LMIIndicationHandler)
        except socket.error, e:
            return False
        self._server._handlers = self._handlers
        self._server_thread = threading.Thread(target=self._server.serve_forever)
        self._server_thread.daemon = True
        if self._certfile:
            self._server.socket = ssl.wrap_socket(
                self._server.socket,
                certfile=self._certfile,
                keyfile=self._keyfile,
                server_side=True)
        self._server_thread.start()
        return True

    def stop(self):
        """
        Stops the indication listener.
        """
        self._server.shutdown()
        self._server_thread.join()

    @property
    def is_alive(self):
        """
        :returns: flag indicating, if the indication listener is running
        :rtype: bool
        """
        return self._server_thread.is_alive()

    @property
    def hostname(self):
        """
        :returns: hostname or address, where the indication listener is waiting for
            messages
        :rtype: string
        """
        return self._hostname

    @property
    def port(self):
        """
        :returns: port, where the indication listener is waiting for messages
        :rtype: int
        """
        return self._port

    def add_handler(self, handler_name_pattern, handler, *args, **kwargs):
        """
        Registers a handler into the indication listener. Returns a string, which is used
        for the indication recognition, when a message arrives.

        :param string handler_name_pattern: string, which may contain set of "**X**"
            characters at the end of the string. The "**X**" characters will be replaced by
            random characters and the whole string will form a unique string.
        :param handler: callable, which will be executed, when a indication is received
        :param tuple args: positional arguments for the handler
        :param dictionary kwargs: keyword arguments for the handler
        :returns: handler unique name
        :rtype: string
        """
        handler_name = self.__create_handler_name(handler_name_pattern + "X" * 16)
        self._handlers[handler_name] = LMIIndicationHandlerCallback(handler, *args, **kwargs)
        return handler_name

    def remove_handler(self, name):
        """
        Removes a specified handler from the indication listener database.

        :param string name: indication name; returned by
            :py:meth:`LMIIndicationListener.add_handler`
        """
        if not name in self._handlers:
            return
        self._handlers.pop(name)



# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from setuptools import setup
from lmi.shell import __version__

setup(
    name="openlmi-tools",
    description="OpenLMI (non)interactive shell",
    version=__version__,
    license="GPLv2+",
    url="http://fedorahosted.org/openlmi/",
    author="Peter Hatina",
    author_email="phatina@redhat.com",
    namespace_packages=['lmi'],
    packages=["lmi", "lmi.shell"],
    install_requires=["openlmi"],
    scripts=["lmishell"]
)



# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from functools import wraps

from LMIUtil import lmi_raise_or_dump_exception

from LMIExceptions import LMIDeletedObjectError

class lmi_return_expr_if_fail(object):
    """
    Decorator, which calls a specified expression and returns its return value instead of
    calling the decorated method, if provided test expression is False; otherwise a method
    is called.

    :param expr_test: expression which determines, if to execute a return
            value expression
    :param expr_ret: expression, which is called, if the ``expr_test`` returns False
    :param expr_ret_args: ``expr_ret`` position arguments
    :param expr_ret_kwargs: ``expr_ret`` keyword arguments
    :param bool Self: flag, which specifies, if to pass ``self`` variable to the ``expr_ret``,
            if ``expr_test`` failed

    Example of usage:

    .. code-block:: python

        class Foo:
            def __init__(self, member):
                self._member = member

            def failed(self):
                print "expression failed"
                return False

            # NOTE: the self parameter to the method call needs to be passed via expr_ret_args,
            # therefore, there is a dummy lambda obj: obj, which is basically self variable.
            @lmi_return_expr_if_fail(lambda obj: obj._member, failed, lambda obj: obj)
            def some_method(self):
                print "some_method called"
                return True

        f = Foo(None)
        f.some_method() == False

        f = Foo(True)
        f.some_method() == True
    """
    def __init__(self, expr_test, expr_ret, Self=False, *expr_ret_args, **expr_ret_kwargs):
        self._expr_test = expr_test
        self._expr_ret = expr_ret
        self._expr_ret_pass_self = Self
        self._expr_ret_args = expr_ret_args
        self._expr_ret_kwargs = expr_ret_kwargs

    def __call__(self, fn):
        """
        Executes a method call, if the test passed.

        :param instancemethod fn: decorated method
        """
        @wraps(fn)
        def wrapper(self_wr, *args, **kwargs):
            failed = False
            try:
                if not self._expr_test(self_wr):
                    failed = True
            except AttributeError, e:
                failed = True
            if failed:
                if self._expr_ret_pass_self:
                    return self._expr_ret(self_wr, *self._expr_ret_args, **self._expr_ret_kwargs)
                return self._expr_ret(*self._expr_ret_args, **self._expr_ret_kwargs)
            return fn(self_wr, *args, **kwargs)
        return wrapper

class lmi_return_val_if_fail(lmi_return_expr_if_fail):
    """
    Decorator, which returns a specified value and no method call is performed, if
    provided test expression is False; otherwise a method is called.

    :param expr_test: if the expression returns False, a method call is called
    :param rval: return value of the method, if the object attribute as expression
        failed

    Example of usage:

    .. code-block:: python

        class Foo:
            def __init__(self, member):
                self._member = member

            @lmi_return_val_if_fail(lambda obj: obj._member, False)
            def some_method(self):
                print "some_method called"
                return True

        f = Foo(None)
        f.some_method() == False

        f = Foo(True)
        f.some_method() == True
    """
    def __init__(self, expr_test, rval):
        lmi_return_expr_if_fail.__init__(self, expr_test, lambda: rval)

class lmi_return_if_fail(lmi_return_val_if_fail):
    """
    Decorator, which returns None and no method call is performed, if provided test
    expression is False; otherwise a method is called.

    :param expr_test: if the expression ``expr_test`` returns True, a method is called

    Example of usage:

    .. code-block:: python

        class Foo:
            def __init__(self, member):
                self._member = member

            @lmi_return_if_fail(lambda obj: obj._member)
            def some_method(self):
                print "some_method called"
                return True

        f = Foo(None)
        f.some_method() == None

        f = Foo(True)
        f.some_method() == True
    """
    def __init__(self, expr_test):
        lmi_return_val_if_fail.__init__(self, expr_test, None)

class lmi_possibly_deleted(lmi_return_expr_if_fail):
    """
    Decorator, which returns None, if provided test expression is True.

    :param expr_ret: callable or return value used, if ``expr_test`` fails
    :param expr_ret_args: expr_ret position arguments
    :param expr_ret_kwargs: expr_ret keyword arguments
    :param bool Self: flag, which specifies, if to pass ``self`` variable to the
        ``expr_ret``, if ``expr_test`` failed

    Example of usage:

    .. code-block:: python

        class Foo:
            def __init__(self, deleted):
                self._deleted = deleted

            @lmi_possibly_deleted(lambda obj: obj._member, lambda: False)
            def some_method(self):
                print "some_method called"
                return True

        f = Foo(None)
        f.some_method() == False

        f = Foo(True)
        f.some_method() == True
    """
    def __init__(self, expr_ret, Self=False, *expr_ret_args, **expr_ret_kwargs):
        call_expr = expr_ret if callable(expr_ret) else lambda: expr_ret
        lmi_return_expr_if_fail.__init__(self, lambda obj: not obj._deleted,
            call_expr, Self=Self, *expr_ret_args, **expr_ret_kwargs)

    def __call__(self, fn):
        """
        Executes a method call, if the test passed. If the test expression returns False,
        specified return value in the constructor is returned. If the LMIShell does not
        dump exceptions, an appropriate exception is raised.

        :param instancemethod fn: decorated method
        :raises: :py:exc:`.LMIDeletedObjectError`
        """
        @wraps(fn)
        def wrapper(self_wr, *args, **kwargs):
            failed = False
            try:
                if not self._expr_test(self_wr):
                    failed = True
            except AttributeError, e:
                failed = True
            if failed:
                errorstr = "This instance has been deleted from a CIM broker"
                lmi_raise_or_dump_exception(LMIDeletedObjectError(errorstr))
                if self._expr_ret_pass_self:
                    return self._expr_ret(self_wr, *self._expr_ret_args, **self._expr_ret_kwargs)
                return self._expr_ret(*self._expr_ret_args, **self._expr_ret_kwargs)
            return fn(self_wr, *args, **kwargs)
        return wrapper

def lmi_class_fetch_lazy(fn):
    """
    Decorator for :py:class:`LMIClass`, which first fetches a wrapped :py:class:`CIMClass`
    object and then executes a wrapped method.

    :param instancemethod fn: wrapped method
    """
    @wraps(fn)
    def wrapped(self, *args, **kwargs):
        if not self._cim_class:
            self.fetch()
        return fn(self, *args, **kwargs)
    return wrapped



# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import sys
import pywbem

from LMIBaseObject import LMIWrapperBaseObject
from LMIUtil import lmi_wrap_cim_instance
from LMIUtil import lmi_wrap_cim_instance_name

class LMIInstanceName(LMIWrapperBaseObject):
    """
    LMI wrapper class representing :py:class:`CIMInstanceName`.

    :param LMIConnection conn: connection object
    :param CIMInstanceName cim_instance_name: wrapped object
    """
    def __init__(self, conn, cim_instance_name):
        super(LMIInstanceName, self).__init__(conn)
        if isinstance(cim_instance_name, LMIInstanceName):
            cim_instance_name = cim_instance_name.wrapped_object
        self._cim_instance_name = cim_instance_name

    def __contains__(self, key):
        """
        :param string key: key name, which will be tested for presence in keybindings
        :returns: True, if the specified key is present in keybindings, False otherwise
        """
        return key in self._cim_instance_name

    def __getattr__(self, name):
        """
        Simplifies the code. Key properties values can be retrieved by
        :samp:`object.key_property`.

        :param string name: class member or key property name
        :returns: class member or key property
        """
        if name in self.__dict__:
            return self.__dict__[name]
        key_props = self.key_properties()
        if key_props and name in key_props:
            member = self._cim_instance_name[name]
            if isinstance(member, pywbem.CIMInstanceName):
                member = lmi_wrap_cim_instance_name(self._conn, member)
            return member
        raise AttributeError(name)

    def __str__(self):
        """
        :returns: string of serialized object
        """
        return "%s(classname=\"%s\", %s)" % ( \
            self.__class__.__name__, \
            self.classname, \
            str(self._cim_instance_name) \
        )

    def __repr__(self):
        """
        :returns: pretty string for the object
        """
        return "%s(classname=\"%s\"...)" % ( \
            self.__class__.__name__, \
            self.classname \
        )

    def copy(self):
        """
        :returns: copy of itself
        """
        return lmi_wrap_cim_instance_name(
                self._conn, self._cim_instance_name.copy())

    def to_instance(self):
        """
        Creates a new :py:class:`LMIInstance` object from :py:class:`LMIInstanceName`.

        :returns: :py:class:`LMIInstance` object if the object was retrieved successfully;
            None otherwise.

        **Usage:** :ref:`instance_names_conversion`.
        """
        (cim_instance, _, errorstr) = self._conn._client._get_instance(
            self._cim_instance_name, LocalOnly=False)
        if not cim_instance:
            return None
        return lmi_wrap_cim_instance(self._conn, cim_instance,
            self._cim_instance_name.classname,
            self._cim_instance_name.namespace
        )

    def key_properties(self):
        """
        :returns: list of strings of key properties

        **Usage:** :ref:`instance_names_key_properties`.
        """
        return self._cim_instance_name.keys()

    def print_key_properties(self):
        """
        Prints out the list of key properties.

        **Usage:** :ref:`instance_names_key_properties`.
        """
        for name in self._cim_instance_name.keys():
            sys.stdout.write("%s\n" % name)

    def key_properties_dict(self):
        """
        :returns: dictionary with key properties and corresponding values
        """
        return self._cim_instance_name.keybindings.copy()

    def key_property_value(self, prop_name):
        """
        :param string prop_name: key property name
        :returns: key property value
        """
        return getattr(self, prop_name)

    @property
    def classname(self):
        """
        :returns: class name
        :rtype: string
        """
        return self._cim_instance_name.classname

    @property
    def namespace(self):
        """
        :returns: namespace name
        :rtype: string
        """
        return self._cim_instance_name.namespace

    @property
    def hostname(self):
        """
        :returns: host name
        :rtype: string
        """
        return self._cim_instance_name.host

    @property
    def wrapped_object(self):
        """
        :returns: wrapped :py:class:`CIMInstanceName` object
        """
        return self._cim_instance_name



# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

class LMIShellCache(object):
    """
    Class representing a LMIShell cache.

    :param bool active: specifies, if the cache is active
    :param list classname_list: list of strings of cached class names
    :param dictionary class_dict: cached :py:class:`CIMClass` objects, where the key is
        the class name and value is :class:`CIMClass` object
    :param dictionary class_superclass_dict: dictionary, where the key is namespace and
        value is dictionary of classname:superclass
    """
    def __init__(self, active=True, classname_list=None, class_dict=None,
            class_superclass_dict=None):
        self._classname_list = classname_list
        self._class_dict = class_dict if not class_dict is None else {}
        self._class_superclass_dict = class_superclass_dict if not class_superclass_dict is None else {}
        self._active = active

    def clear(self):
        """
        Clears the cache.
        """
        self._classname_list = None
        self._class_dict = {}
        self._class_superclass_dict = {}

    def get_classes(self):
        """
        :returns: list of cached class names
        """
        return self._classname_list

    def set_classes(self, classname_list):
        """
        Stores a new class names' list.
        """
        self._classname_list = classname_list

    def get_class(self, classname):
        """
        :param string classname: cached class name
        :returns: cached :py:class:`CIMClass` object, if proper class name provided, None otherwise
        """
        if not classname in self._class_dict:
            return None
        return self._class_dict[classname]

    def add_class(self, cim_class):
        """
        Stores a new :py:class:`CIMClass` object into the cache.

        :param CIMClass cim_class: :py:class:`CIMClass` object
        """
        self._class_dict[cim_class.classname] = cim_class

    def has_superclass(self, classname, namespace):
        """
        :param string classname: cached class name
        :param string namespace: namespace name
        :returns: True, if the cache contains superclass to the given class name; False otherwise
        """
        if not namespace in self._class_superclass_dict:
            return False
        if not classname in self._class_superclass_dict[namespace]:
            return False
        return True

    def get_superclass(self, classname, namespace):
        """
        :param string classname: cached class name
        :param string namespace: namespace name
        :returns: cached superclass to the given class name
        :rtype: string
        """
        if not self.has_superclass(classname, namespace):
            return None
        return self._class_superclass_dict[namespace][classname]

    def add_superclass(self, classname, superclass, namespace):
        """
        Stores a new pair classname : superclassname into the cache.

        :param string classname: class name to be stored
        :param string superclass: super class name to be stored
        :param string namespace: namespace name of the classname
        """
        if not namespace in self._class_superclass_dict:
            self._class_superclass_dict[namespace] = {}
        self._class_superclass_dict[namespace][classname] = superclass

    @property
    def active(self):
        """
        :returns: True, if the cache is active; False otherwise
        """
        return self._active

    @active.setter
    def active(self, val):
        """
        Property setter for the property active.
        """
        self._active = val
